// JavaScript code for the BLE Peripheral example app.

var base64 = cordova.require('cordova/base64');

// Application object.
var app = {};

// Device list.
app.devices = {};

// UI methods.
app.ui = {};

function initialize()
{
	document.addEventListener('deviceready', this.onDeviceReady, false);

}

function onDeviceReady()
{	
	startServer(startAdvert());
}

function startServer(win)
{
	var settings = {
		onConnectionStateChange:function(deviceHandle, connected) {
			hyper.log("device "+deviceHandle+" "+(connected?"":"dis")+"connected");
		},
		services:[
			{
				// matching advert data.
				uuid:"00002220-0000-1000-8000-00805f9b34fb", //0000dead-0000-1000-8000-00805f9b34fb 
				type: 0,
				characteristics:[
					{
						// random uuid.
						uuid:"00002222-0000-1000-8000-00805f9b34fb", //  0000dea1-0000-1000-8000-00805f9b34fb
						handle:1,
						permissions:1|16, //PERMISSION_READ | PERMISSION_WRITE
						properties:2|8|16, // PROPERTY_READ | PROPERTY_WRITE | PROPERTY_NOTIFY
						writeType:2, // WRITE_TYPE_DEFAULT
						onReadRequest:function(deviceHandle, requestId) {
							hyper.log("hallo");
							var data = new Uint8Array([]);
							hyper.log("cr1 "+requestId+": "+evothings.util.typedArrayToHexString(data));
							evothings.ble.sendResponse(deviceHandle, requestId, data, function() {
								hyper.log("cr1 success");
							}, function(err) {
								hyper.log("cr1 fail: "+err);
							});
						},
						onWriteRequest:function(deviceHandle, requestId, data) {
							hyper.log("cw1 "+requestId+":"+evothings.util.typedArrayToHexString(data));
							evothings.ble.sendResponse(deviceHandle, requestId, null, function() {
								hyper.log("cw1 success");
							}, function(err) {
								hyper.log("cw1 fail: "+err);
							});
						},
						descriptors:[
							// notification control.
							(function() {
								var value = new Uint8Array([0,0]);
								return {
									uuid:'00002902-0000-1000-8000-00805f9b34fb',
									permissions:1|16,
									onReadRequest:function(deviceHandle, requestId) {
										hyper.log("dr1 "+requestId);
										evothings.ble.sendResponse(deviceHandle, requestId, value, function() {
											hyper.log("dr1 success");
										}, function(err) {
											hyper.log("dr1 fail: "+err);
										});
									},
									onWriteRequest:function(deviceHandle, requestId, data) {
										hyper.log("dw1 "+requestId+": "+evothings.util.typedArrayToHexString(data));
										value = data;
										evothings.ble.sendResponse(deviceHandle, requestId, value, function() {
											hyper.log("dw1 success");
										}, function(err) {
											hyper.log("dw1 fail: "+err);
										});
									},
								}
							})(),
							// characteristic description.
							{
								uuid:'00002901-0000-1000-8000-00805f9b34fb',
								permissions:1 , //READ, 
								onReadRequest:function(deviceHandle, requestId) {
									var data = evothings.ble.toUtf8("customThing");
									hyper.log("dr3 "+requestId+": "+evothings.util.typedArrayToHexString(data));
									evothings.ble.sendResponse(deviceHandle, requestId, data, function() {
										hyper.log("dr3 success");
									}, function(err) {
										hyper.log("dr3 fail: "+err);
									});
								},
								// no write permission here. discard any incoming writes.
								onWriteRequest:function(deviceHandle, requestId, data) {
									hyper.log("dw3 "+requestId+":"+evothings.util.typedArrayToHexString(data));
									evothings.ble.sendResponse(deviceHandle, requestId, data, function() {
										hyper.log("dw3 success");
									}, function(err) {
										hyper.log("dw3 fail: "+err);
									});
								},
							},
							// random uuid.
							(function() {
								var value = new Uint8Array([1,2,3,4]);
								return {
									uuid:'0000dea2-0000-1000-8000-00805f9b34fb',
									permissions:1|16,
									onReadRequest:function(deviceHandle, requestId) {
										hyper.log("dr2 "+requestId);
										evothings.ble.sendResponse(deviceHandle, requestId, value, function() {
											hyper.log("dr2 success");
										}, function(err) {
											hyper.log("dr2 fail: "+err);
										});
									},
									onWriteRequest:function(deviceHandle, requestId, data) {
										hyper.log("dw2 "+requestId+":"+evothings.util.typedArrayToHexString(data));
										value = data;
										evothings.ble.sendResponse(deviceHandle, requestId, null, function() {
											hyper.log("dw2 success");
										}, function(err) {
											hyper.log("dw2 fail: "+err);
										});
									},
								}
							})(),
						],	//descriptors
					},
				],	//characteristics
			},
		],	//services
	};

	evothings.ble.peripheral.stopGattServer(function() {
		hyper.log("GATT server already started!?!");
	}, function(err) {
		hyper.log(err+", as expected");
		evothings.ble.peripheral.startGattServer(settings, function() {
			hyper.log("GATT server started successfully.");
			win();
		}, function(err) {
			hyper.log("GATT server start error: "+err);
		});
	});
}

function startAdvert()
{
	stopAdvert();

	var data = {
		includeDeviceName:true,
		//includeTxPowerLevel:true,
		//serviceUUIDs:["00001234-0000-1000-8000-00805f9b34fb", "5678abcd-0000-1000-8000-00805f9b34fb"],
		serviceUUIDs:["00002220-0000-1000-8000-00805f9b34fb"],
		//serviceData:{"7d444841-9dc0-11d1-b245-5ffdce74fad2":base64.fromArrayBuffer(new Uint8Array([6,7,8,9,0]))},
		manufacturerData:{"42":base64.fromArrayBuffer(new Uint8Array([1,2,3,4,5]))},
	};

	var scanRespData = {
		//includeDeviceName:false,
		//includeTxPowerLevel:true,
		serviceUUIDs:["deaddead-0000-1000-8000-00805f9b34fb"],
		//serviceData:{"7d444841-9dc0-11d1-b245-5ffdce74fad2":base64.fromArrayBuffer(new Uint8Array([6,7,8,9,0]))},
		serviceData:{"0000dead-0000-1000-8000-00805f9b34fb":base64.fromArrayBuffer(new Uint8Array([6,7,8,9,0]))},
		//manufacturerData:{"42":base64.fromArrayBuffer(new Uint8Array([1,2,3,4,5]))},
	};

	var settings = {
		advertiseMode:"ADVERTISE_MODE_BALANCED",
		connectable:true,
		//timeoutMillis:0,
		txPowerLevel:"ADVERTISE_TX_POWER_HIGH",
		broadcastData:data,
		//scanResponseData:scanRespData,
	};

	evothings.ble.peripheral.startAdvertise(settings,
		function()
		{
			hyper.log("Advertise started.");
		},
		function(errorCode)
		{
			hyper.log("startAdvertise failed: "+errorCode);
		}
	);
}

// Stop scanning for devices.
function stopAdvert()
{
	evothings.ble.peripheral.stopAdvertise();
}