/**************************************************************************
# 	NAME: emulator.evothings.bluetooth.js
# 	AUTHOR: Hans Magnus Hallaråker
#	CONTRIBUTOR: 
#
**************************************************************************/

"use strict";

/*
 * anyboard-emulator-token
 *
 *
 * Dependencies:
 *      AnyBoard (...)
 *      cordova.js (https://cordova.apache.org/)
 *      evothings.ble (https://github.com/evothings/cordova-ble)
 */

(function(){
    var emulator = new AnyBoard.Driver({
        name: 'anyboard-rfduino-token',
        description: 'emulator',
        dependencies: 'evothings.ble',
        version: '0.1',
        date: '2017-05-03',
        type: ['bluetooth'],
        compatibility: [
            {
            }
        ]
    });

    emulator.scan = function(argument) {
        // body...
    }
    emulator.connect = function() {
        // body...
    }

    emulator.parse = function(command, uint8array) {

        switch (command) {
            case emulator._CMD_CODE.GET_BATTERY_STATUS:
                for (var i = 1; i < uint8array.length; i++)
                    strData += String.fromCharCode(uint8array[i])
                token.trigger('GET_BATTERY_STATUS', {"value": strData});
                break;
            case emulator._CMD_CODE.MOVE:
                var currentTile = uint8array[1];
                var previousTile = uint8array[2];
                token.trigger('MOVE', {"value": currentTile, "newTile": currentTile, "oldTile": previousTile}); //this is the one in use
                token.trigger('MOVE_TO', {'meta-eventType': 'token-constraint' ,"constraint": currentTile});
                token.trigger('MOVE_FROM', {'meta-eventType': 'token-constraint' ,"constraint": previousTile});
                token.currentTile = currentTile;
                break;
            case emulator._CMD_CODE.TTEVENT:
                token.trigger('TTEVENT', {'meta-eventType': 'token-token',"token": this});
                break;
            case emulator._CMD_CODE.GET_NAME:
                for (var i = 1; i < uint8array.length; i++)
                    strData += String.fromCharCode(uint8array[i])
                token.trigger('GET_NAME', {"value": strData});
                break;
            case emulator._CMD_CODE.GET_VERSION:
                for (var i = 1; i < uint8array.length; i++)
                    strData += String.fromCharCode(uint8array[i])
                token.trigger('GET_VERSION', {"value": strData});
                break;
            case emulator._CMD_CODE.GET_UUID:
                for (var i = 1; i < uint8array.length; i++)
                    strData += String.fromCharCode(uint8array[i])
                token.trigger('GET_UUID', {"value": strData});
                break;
            case emulator._CMD_CODE.LED_BLINK:
                token.trigger('LED_BLINK');
                break;
            case emulator._CMD_CODE.LED_OFF:
                token.trigger('LED_OFF');
                break;
            case emulator._CMD_CODE.LED_ON:
                token.trigger('LED_ON');
                break;
            case emulator._CMD_CODE.HAS_LED:
                token.trigger('HAS_LED', {"value": uint8array[1]})
                break;
            case emulator._CMD_CODE.HAS_LED_COLOR:
                token.trigger('HAS_LED_COLOR', {"value": uint8array[1]})
                break;
            case emulator._CMD_CODE.HAS_VIBRATION:
                token.trigger('HAS_VIBRATION', {"value": uint8array[1]})
                break;
            case emulator._CMD_CODE.HAS_COLOR_DETECTION:
                token.trigger('HAS_COLOR_DETECTION', {"value": uint8array[1]})
                break;
            case emulator._CMD_CODE.HAS_LED_SCREEN:
                token.trigger('HAS_LED_SCREEN', {"value": uint8array[1]})
                break;
            case emulator._CMD_CODE.HAS_RFID:
                token.trigger('HAS_RFID', {"value": uint8array[1]})
                break;
            case emulator._CMD_CODE.HAS_NFC:
                token.trigger('HAS_NFC', {"value": uint8array[1]})
                break;
            case emulator._CMD_CODE.HAS_ACCELEROMETER:
                token.trigger('HAS_ACCELEROMETER', {"value": uint8array[1]})
                break;
            case emulator._CMD_CODE.HAS_TEMPERATURE:
                token.trigger('HAS_TEMPERATURE', {"value": uint8array[1]})
                break;
            case emulator._CMD_CODE.VIBRATE:
                token.trigger('VIBRATE')
                break;
            case emulator._CMD_CODE.COUNT:
                token.trigger('COUNT')
                break;
            case emulator._CMD_CODE.DISPLAY_X:
                token.trigger('DISPLAY_X')
                break;
            case emulator._CMD_CODE.DISPLAY_W:
                token.trigger('DISPLAY_W')
                break;
            case emulator._CMD_CODE.DISPLAY_UP:
                token.trigger('DISPLAY_UP')
                break;
            case emulator._CMD_CODE.DISPLAY_DOWN:
                token.trigger('DISPLAY_DOWN')
                break;
            case emulator._CMD_CODE.TILT:
                token.trigger('TILT', {'meta-eventType': 'token'});
                break;
            case emulator._CMD_CODE.TAP:
                token.trigger('TAP', {'meta-eventType': 'token'});
                break;
            case emulator._CMD_CODE.DOUBLE_TAP:
                token.trigger('DOUBLE_TAP', {'meta-eventType': 'token'});
                break;
            case emulator._CMD_CODE.SHAKE:
                token.trigger('SHAKE', {'meta-eventType': 'token'});
                break;
            case emulator._CMD_CODE.DISPLAY_DIGIT:
                token.trigger('DISPLAY_DIGIT', {"value": uint8array[1]});
                break;
            default:
                token.trigger('INVALID_DATA_RECEIVE', {"value": uint8array});
        }

    }
    

    /* Internal mapping from commands to uint8 representations */
    emulator._CMD_CODE = {
        MOVE: 194,
		TTEVENT: 195,
        GET_NAME: 32,
        GET_VERSION: 33,
        GET_UUID: 34,
        GET_BATTERY_STATUS: 35,
        LED_OFF: 128,
        LED_ON: 129,
        LED_BLINK: 130,
        HAS_LED: 64,
        HAS_LED_COLOR: 65,
        HAS_VIBRATION: 66,
        HAS_COLOR_DETECTION: 67,
        HAS_LED_SCREEN: 68,
        HAS_RFID: 71,
        HAS_NFC: 72,
        HAS_ACCELEROMETER: 73,
        HAS_TEMPERATURE: 74,
		VIBRATE: 200,
		TAP: 201,
		DOUBLE_TAP: 202,
		SHAKE: 203,
		TILT: 204,
		COUNT: 205,
		DISPLAY_X: 206,
        DISPLAY_DIGIT:207,
        DISPLAY_W: 208,
        DISPLAY_UP: 209,
        DISPLAY_DOWN: 210
    };

    /* Internal mapping between color strings to Uint8 array of RGB colors */
    emulator._COLORS = {
        'red': new Uint8Array([255, 0, 0]),
        'green': new Uint8Array([0, 255, 0]),
        'blue': new Uint8Array([0, 0, 255]),
        'white': new Uint8Array([255, 255, 255]),
        'pink': new Uint8Array([255, 0, 255]),
        'yellow': new Uint8Array([255, 255, 0]),
        'aqua': new Uint8Array([0, 255, 255]),
        'off': new Uint8Array([0, 0, 0])
    };

    /* Internal mapping and generation of commands */
    var NO_PARAMS = false;
    var HAS_PARAMS = true;
    var USE_CACHE = true;
    emulator._COMMANDS = {
        GET_NAME: emulator.parse(
            "GET_NAME",
            emulator._CMD_CODE.GET_NAME,
            NO_PARAMS,
            USE_CACHE),
        GET_VERSION: emulator.parse(
            "GET_VERSION",
            emulator._CMD_CODE.GET_VERSION,
            NO_PARAMS,
            USE_CACHE),
        GET_UUID: emulator.parse(
            "GET_UUID",
            emulator._CMD_CODE.GET_UUID,
            NO_PARAMS,
            USE_CACHE),
        GET_BATTERY_STATUS: emulator.parse(
            "GET_BATTERY_STATUS",
            emulator._CMD_CODE.GET_BATTERY_STATUS,
            NO_PARAMS),
        LED_OFF: emulator.parse(
            "LED_OFF",
            emulator._CMD_CODE.LED_OFF,
            NO_PARAMS),
        LED_ON: emulator.parse(
            "LED_ON",
            emulator._CMD_CODE.LED_ON,
            HAS_PARAMS),
        LED_BLINK: emulator.parse(
            "LED_BLINK",
            emulator._CMD_CODE.LED_BLINK,
            HAS_PARAMS),
        HAS_LED: emulator.parse(
            "HAS_LED",
            emulator._CMD_CODE.HAS_LED,
            NO_PARAMS,
            USE_CACHE),
        HAS_LED_COLOR: emulator.parse(
            "HAS_LED_COLOR",
            emulator._CMD_CODE.HAS_LED_COLOR,
            NO_PARAMS,
            USE_CACHE),
        HAS_VIBRATION: emulator.parse(
            "HAS_VIBRATION",
            emulator._CMD_CODE.HAS_VIBRATION,
            NO_PARAMS,
            USE_CACHE),
        HAS_COLOR_DETECTION: emulator.parse(
            "HAS_COLOR_DETECTION",
            emulator._CMD_CODE.HAS_COLOR_DETECTION,
            NO_PARAMS,
            USE_CACHE),
        HAS_LED_SCREEN: emulator.parse(
            "HAS_LED_SCREEN",
            emulator._CMD_CODE.HAS_LED_SCREEN,
            NO_PARAMS,
            USE_CACHE),
        HAS_RFID: emulator.parse(
            "HAS_RFID",
            emulator._CMD_CODE.HAS_RFID,
            NO_PARAMS,
            USE_CACHE),
        HAS_NFC: emulator.parse(
            "HAS_NFC",
            emulator._CMD_CODE.HAS_NFC,
            NO_PARAMS,
            USE_CACHE),
        HAS_ACCELEROMETER: emulator.parse(
            "HAS_ACCELEROMETER",
            emulator._CMD_CODE.HAS_ACCELEROMETER,
            NO_PARAMS,
            USE_CACHE),
        HAS_TEMPERATURE: emulator.parse(
            "HAS_TEMPERATURE",
            emulator._CMD_CODE.HAS_TEMPERATURE,
            NO_PARAMS,
            USE_CACHE),
		VIBRATE: emulator.parse(
            "VIBRATE",
            emulator._CMD_CODE.VIBRATE,
            HAS_PARAMS),
		COUNT: emulator.parse(
            "COUNT",
            emulator._CMD_CODE.COUNT,
            HAS_PARAMS),
		DISPLAY_X: emulator.parse(
            "DISPLAY_X",
            emulator._CMD_CODE.DISPLAY_X,
            HAS_PARAMS),
        DISPLAY_DIGIT: emulator.parse(
            "DISPLAY_DIGIT",
            emulator._CMD_CODE.DISPLAY_DIGIT,
            HAS_PARAMS),
        DISPLAY_W: emulator.parse(
            "DISPLAY_W",
            emulator._CMD_CODE.DISPLAY_W,
            HAS_PARAMS),
        DISPLAY_UP: emulator.parse(
            "DISPLAY_UP",
            emulator._CMD_CODE.DISPLAY_UP,
            HAS_PARAMS),
        DISPLAY_DOWN: emulator.parse(
            "DISPLAY_DOWN",
            emulator._CMD_CODE.DISPLAY_DOWN,
            HAS_PARAMS)
    };



    emulator.getName = function (token, win, fail) {
        this._COMMANDS.GET_NAME(token, win, fail);
    };

    emulator.getVersion = function (token, win, fail) {
        this._COMMANDS.GET_VERSION(token, win, fail);
    };

    emulator.getUUID = function (token, win, fail) {
        this._COMMANDS.GET_UUID(token, win, fail);
    };

    emulator.hasLed = function(token, win, fail) {
        this._COMMANDS.HAS_LED(token, win, fail);
    };

    emulator.hasLedColor = function(token, win, fail) {
        this._COMMANDS.HAS_LED_COLOR(token, win, fail);
    };

    emulator.hasVibration = function(token, win, fail) {
        this._COMMANDS.HAS_VIBRATION(token, win, fail);
    };

    emulator.hasColorDetection = function(token, win, fail) {
        this._COMMANDS.HAS_COLOR_DETECTION(token, win, fail);
    };

    emulator.hasLedScreen = function(token, win, fail) {
        this._COMMANDS.HAS_LED_SCREEN(token, win, fail);
    };

    emulator.hasRfid = function(token, win, fail) {
        this._COMMANDS.HAS_RFID(token, win, fail);
    };

    emulator.hasNfc = function(token, win, fail) {
        this._COMMANDS.HAS_NFC(token, win, fail);
    };

    emulator.hasAccelometer = function(token, win, fail) {
        this._COMMANDS.HAS_ACCELEROMETER(token, win, fail);
    };

    emulator.hasTemperature = function(token, win, fail) {
        this._COMMANDS.HAS_TEMPERATURE(token, win, fail);
    };

    emulator.ledOn = function (token, value, win, fail) {
        value = value || 'white';

        if (typeof value === 'string' && value in this._COLORS) {
            emulator.ledOn(token, this._COLORS[value], win, fail);
        } else if ((value instanceof Array || value instanceof Uint8Array) && value.length === 3) {
            this._COMMANDS.LED_ON(token, new Uint8Array([value[0], value[1], value[2]]), win, fail);
        } else {
            fail && fail('Invalid or unsupported color parameters');
        }
    };

    emulator.vibrate = function (token, value, win, fail) {
        this._COMMANDS.VIBRATE(token, new Uint8Array(value), win, fail);
    };

	emulator.count = function (token, value, win, fail) {
        this._COMMANDS.COUNT(token, new Uint8Array(value), win, fail);
    };

	emulator.displayX = function (token, value, win, fail) {
        this._COMMANDS.DISPLAY_X(token, new Uint8Array(value), win, fail);
    };
    emulator.displayW = function (token, value, win, fail) {
        this._COMMANDS.DISPLAY_W(token, new Uint8Array(value), win, fail);
    };
    emulator.displayUp = function (token, value, win, fail) {
        this._COMMANDS.DISPLAY_UP(token, new Uint8Array(value), win, fail);
    };
    emulator.displayDown = function (token, value, win, fail) {
        this._COMMANDS.DISPLAY_DOWN(token, new Uint8Array(value), win, fail);
    };
    emulator.displayDigit = function (token, value, win, fail) {
        this._COMMANDS.DISPLAY_DIGIT(token, new Uint8Array(value), win, fail);
    };

    emulator.ledBlink = function (token, value, win, fail) {
        value = value || 'white';

        if (typeof value === 'string' && value in this._COLORS) {
            emulator.ledBlink(token, this._COLORS[value], win, fail);
        } else if ((value instanceof Array || value instanceof Uint8Array) && value.length === 3) {
            this._COMMANDS.LED_BLINK(token, new Uint8Array([value[0], value[1], value[2], 30]), win, fail);
        } else {
            fail && fail('Invalid or unsupported color parameters');
        }
    };

    emulator.ledOff = function (token, win, fail) {
        this._COMMANDS.LED_OFF(token, win, fail);
    };

    /*emulator.onColorChange = function(token, changeCallback) {
        token.on('colorChange', changeCallback);
    };*/

})();
